<?php

// Author: oxana.smirnova@quark.lu.se

require_once('headfoot.inc');
require_once('lmtable.inc');
require_once('cnvtime.inc');
require_once('cnvname.inc');
require_once('comfun.inc');
require_once('toreload.inc');
require_once('ldap_nice_dump.inc');
require_once('recursive_giis_info.inc');
require('ldap_purge.inc');

// Setting up the page itself

$toppage = new LmDoc("userlist");

/**
 * @return int
 * @param bdn string
 * @param owner string
 * @desc Retrieves the list of user's jobs
 */
function do_userlist ($bdn="mds-vo-name=NorduGrid,o=Grid",$owner)
{
  /**
   * FUNCTION DO_USERLIST (string bdn, string owner)
   * Retrieves the user jobs information from the NorduGrid
   * Uses LDAP functions of PHP
   *
   * Author: O.Smirnova (June 2002)
   * inspired by the LDAPExplorer by T.Miao
   *
   * input:
   * bdn (base DN, def. mds-vo-name=NordiGrid,o=Grid)
   * owner (DN of a user)
   *
   * output:
   * an HTML table, containing:
   * per each cluster:
   * - list of jobs
   */
	
  include('settings.inc');
	
  // Array defining the attributes to be returned
	
  $lim  = array( "dn", USR_USSN, USR_CPUS, USR_QUEU, USR_DISK );
  $ulim = array( "dn", JOB_NAME, JOB_EQUE, JOB_ECLU, JOB_GOWN, JOB_SUBM, JOB_STAT, JOB_USET, JOB_ERRS );
	
  $tlim = 20;
  $tout = 20;

  $uname  = rawurldecode($owner);
  $family = cnvname($uname);
	
  // ldapsearch filter string for jobs
	
  $filter  = "(&(objectclass=".OBJ_USER.")(".USR_USSN."=$uname))";
  $ufilter = "(&(objectclass=".OBJ_AJOB.")(".JOB_GOWN."=$uname))";
	
  // Header table
	
  $toptable = new LmTableTop("","Information for $family");

  $gentries = recursive_giis_info($giislist,"cluster",0);
  $nc       = count($gentries);
  if ( !$nc ) {
    // NO SITES FOUND!
    echo "<BR><font color=red><B>Information System is empty</B><br>(None of the top level GIIS returned information)</font>\n";
    return 1;
  }

  $dsarray = array ();
  $hnarray = array ();
  $sitetag = array (); /* a tag to skip duplicated entries */

  for ( $k = 0; $k < $nc; $k++ ) {
    $clhost = $gentries[$k]["host"];
    $clport = $gentries[$k]["port"];

    $clconn = ldap_connect($clhost,$clport);
    if ( $clconn && !$sitetag[$clhost] ) {
      array_push($dsarray,$clconn);
      array_push($hnarray,$clhost);
      $sitetag[$clhost] = 1; /* filtering tag */
    }
  }

  $nhosts = count($dsarray);
  if ( !$nhosts ) {
    // NO SITES REPLY...
    echo "<BR><font color=red><B>Information System is empty</B><br>(None of the GRIS returned connection)</font>\n";
    return 1;
  }

  // Search all clusters for (a) allowed queues and (b) for user jobs

  $uiarray = @ldap_search($dsarray,DN_LOCAL,$filter,$lim,0,0,$tlim,LDAP_DEREF_NEVER,$tout);
  $srarray = @ldap_search($dsarray,DN_LOCAL,$ufilter,$ulim,0,0,$tlim,LDAP_DEREF_NEVER,$tout);
           
  // Loop on results: first go queues

  // HTML table initialisation
  $utable   = new LmTable("userres");
  $urowcont = array();
  
  for ( $ids = 0; $ids < $nhosts; $ids++ ) {

    $ui    = $uiarray[$ids];
    $hn    = $hnarray[$ids];
    $dst   = $dsarray[$ids];
    $curl  = popup("clusdes.php?host=$hn&port=2135",700,620,1);

    if ($dst && $ui) {

      $nqueues = ldap_count_entries($dst,$ui);
      if ($nqueues > 0) {
				
	// If there are valid entries, tabulate results
				
	$allres  = ldap_get_entries($dst,$ui);
	$results = ldap_purge($allres);
	$nqueues = $allres["count"];
				
	//   define("CMPKEY",USR_CPUS);
	//   usort($allres,"ldap_entry_comp");
				
	// loop on queues
				
	for ($j=0; $j<$nqueues; $j++) {
	  $parts = ldap_explode_dn($allres[$j]["dn"],0);
	  foreach ($parts as $part) {
	    $pair = explode("=",$part);
	    switch ( $pair[0] ) {
	    case CLU_NAME:
	      $ucluster = $pair[1];
	      break;
	    case QUE_NAME:
	      $uqueue   = $pair[1];
	      break;
	    }
	  }
	  $qurl  = popup("quelist.php?host=$ucluster&port=2135&qname=$uqueue",750,430,6);
	  $curl  = popup("clusdes.php?host=$ucluster&port=2135",700,620,1);
	  $fcpu  = $allres[$j][USR_CPUS][0];
	  $fdisk = $allres[$j][USR_DISK][0];
	  $exque = $allres[$j][USR_QUEU][0];
	  $urowcont[] = "<a href=\"$curl\">$ucluster</a>:<a href=\"$qurl\">$uqueue</a>";
	  $urowcont[] = $fcpu;
	  $urowcont[] = $exque;
	  $urowcont[] = $fdisk;
	  $utable->addrow($urowcont);
	  $urowcont = array();
	}
      } else {
	$utable->adderror("<B>Not authorised at host <a href=\"$curl\">$hn</a></B>");
      }
    } else {
      $utable->adderror("<B>Host <a href=\"$curl\">$hn</a> does not answer</B>");
    }
    ldap_free_result($ui);
  }
  $utable->close();
  echo "<BR>\n";

  // HTML table initialisation
  $jtable  = new LmTable("userlist");
  $rowcont = array();
  $jcount  = 0;
				
  for ( $ids = 0; $ids < $nhosts; $ids++ ) {

    $sr  = $srarray[$ids];
    $dst = $dsarray[$ids];
    if ($dst && $sr) {
			
      // If search returned, check that there are valid entries
			
      $nmatch = ldap_count_entries($dst,$sr);
      if ($nmatch > 0) {
	// If there are valid entries, tabulate results
				
	$allentries = ldap_get_entries($dst,$sr);
	$entries    = ldap_purge($allentries);
	$njobs      = $entries["count"];
				
	define("CMPKEY",JOB_SUBM);
	usort($entries,"ldap_entry_comp");
				
	// loop on jobs
				
	for ($i=1; $i<$njobs+1; $i++) {
	  $jobdn   = rawurlencode($entries[$i]["dn"]);
	  $curstat = $entries[$i][JOB_STAT][0];
	  $stahead = substr($curstat,0,12);
	  if ($stahead=="FINISHED at:") {
	    $ftime   = substr(strrchr($curstat, " "), 1);
	    $ftime   = cnvtime($ftime);
	    $curstat = "FINISHED at: ".$ftime;
	  }
	  $jobname  = ($entries[$i][JOB_NAME][0]) ? $entries[$i][JOB_NAME][0] : "<font color=red>N/A</font>";
	  $queue    = ($entries[$i][JOB_EQUE][0]) ? $entries[$i][JOB_EQUE][0] : "N/A";
	  $cluster  = ($entries[$i][JOB_ECLU][0]) ? $entries[$i][JOB_ECLU][0] : "N/A";
	  $time     = ($entries[$i][JOB_USET][0]) ? $entries[$i][JOB_USET][0] : "N/A";
	  $error    = ($entries[$i][JOB_ERRS][0]);
	  if ( $error ) $error = ( preg_match("/user/i",$error) ) ? "X" : "!";
	  $newwin   = popup("jobstat.php?host=$cluster&port=2135&status=$status&jobdn=$jobdn",750,430,4);
	  $quewin   = popup("quelist.php?host=$cluster&port=2135&qname=$queue",750,430,6);
	  $clstring = popup("clusdes.php?host=$cluster&port=2135",700,620,1);
	  $jcount++;
					
	  // filling the table
					
	  $rowcont[] = "$jcount&nbsp;<b><font color=red>$error</font></b>";
	  $rowcont[] = "<a href=\"$newwin\">$jobname</a>";
	  $rowcont[] = "$curstat";
	  $rowcont[] = "$time";
	  $rowcont[] = "<a href=\"$clstring\">$cluster</a>";
	  $rowcont[] = "<a href=\"$quewin\">$queue</a>";
	  $jtable->addrow($rowcont);
	  $rowcont = array();
	}
      }
      //      else {
      //	$jtable->adderror("<B>No recent jobs by $family found</B>");
      //      }
    }
    //    else {
    //      $jtable->adderror("<B>No information about $family found</B>");
    //    }
    ldap_free_result($sr);
  }
  if ( !$jcount ) $jtable->adderror("<B>No recent jobs by $family found</B>");
  $jtable->close();

  return 0;
}

$owner = $_GET["owner"];
do_userlist(DN_GLOBL,$owner);

// Done

$toppage->close();

?>
